from PyQt5 import QtCore, QtWidgets, QtGui
from PyQt5.QtCore import Qt, QMimeData
from PyQt5.QtWidgets import QTreeWidgetItem, QFrame, QFileDialog, QMessageBox
from PyQt5.QtGui import QDrag, QFont

from decode import decode_data, get_rows

class List(QtWidgets.QListWidget):

    def __init__(self, name, parent=None):
        super().__init__(parent)

        self.parent = parent

        self.setObjectName(name)
        self.name = name

        self.setDragDropMode(self.DragDrop)
        self.setSelectionMode(self.ExtendedSelection)
        self.setAcceptDrops(True)

        self.setDefaultDropAction(QtCore.Qt.MoveAction)

        #font = QFont("Consolas", 10)
        font = QFont("Consolas", 7)
        self.setFont(font)

        self.setMinimumSize(QtCore.QSize(210, 100))

        self.thing = 0

        self.itemSelectionChanged.connect(self.items_selected)


    def items_selected(self):
        #print(f"items_selected: {self.name} : {self.selectedItems()}")
        ###self.parent.set_selected_box(self.name)
        for i in self.selectedItems():
            i.setSelected(True)

    def clear_items(self):
        items = self.selectedItems()
        #print(f"Lists:List clear_items : {items}")
        #for i in self.selectedItems():
        n = []
        for i in items:
            #print(f"Lists:List clear_items : index = {self.indexFromItem(i).row()}")
            n.append(self.indexFromItem(i).row())
        n.reverse()
        #print(f"Lists:list : n = {n}")
        for i in n:
            self.takeItem(i)


    def get_item_at(self, row):
        return self.selectedItems()[row]


    def get_thing(self):
        return self.thing


    def dragEnterEvent(self, event):
        md = event.mimeData()
        #print(f"mimeData text = '{event.mimeData().text()}'")
        bytearray = event.mimeData().data('application/x-qabstractitemmodeldatalist')
        data_items = decode_data(bytearray)
        data_item = data_items[0][0]
        #print("Drag ENTER")
        #print(f"data_items = {data_items[0][0]}")
        #print(f"data_item = {data_item.typeName()}")
        #print(f"data_item = {data_item.userType()}")
        #print(f"data_item = {data_item.value()}")
        #print(f'data_item = {data_item}')
        thing = data_item.value()
        #print(F"THING: {thing}")
        #print(f"data_item = {thing.parent()}")
        nitems = len(data_items)
        total_items = nitems + self.count()
        #if self.count() + nitems < 13:
        if self.count() + nitems < 17:
            #drop_completed = True
            event.accept()
            if self.parent != None:
                self.parent.parent.statusbar.showMessage(f"")
        else:
            event.ignore()
            if self.parent != None:
                self.parent.parent.statusbar.showMessage(f"Too many parameters: {total_items}")

    def dropEvent(self, event):
        if event.source() == self:
            event.ignore
            return
        """
        if event.mimeData().hasFormat('application/x-qabstractitemmodeldatalist'):
            data = event.mimeData()
            source_item = QtGui.QStandardItemModel()
            source_item.dropMimeData(data, QtCore.Qt.CopyAction, 0,0, QtCore.QModelIndex())
            print(source_item.item(0, 0).text())
            """
        
        super().dropEvent(event)
        #for i in range(self.count()):
            #print(f'parent = {self.item(i)}')
        #print(f"list has {self.count()} items")
        #for i in range(self.count()):
            #print(f"{self.item(i).text()}")
        ##print(event.mimeData().text())


class ListItem(QtWidgets.QListWidgetItem):
    def __init__(self, name, id, src, unit, row):
        super().__init__()
        self.name = name
        text = '{:25}'.format(name) + '{:12}'.format(id) + '{:14}'.format(src) + '{:2}'.format(unit)
        self.setText(text)
        self.id = id
        self.src = src
        self.unit = unit
        self.row = row
        self.main_tab = 10     # Unique tab name
        self.sub_tab = 10     # Unique tab name
        

    def id(self):
        return self.id


    def get_components(self):
        return(self.name, self.id, self.src, self.unit, self.row)


    def row(self):
        '''
        Returns the row (starting from 0) of the selected item.
        '''
        return self.row

    def setSelectable(self):
        pass

    def select(self):
        self.setSelected(True)

    def deselect(self):
        self.setSelected(False)


    def disable(self):
        self.setFlags(Qt.NoItemFlags)


    def enable(self):
        self.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)


class ChildListItem(ListItem):
    def __init__(self, name, id, src, unit, row):
        super(ChildListItem, self).__init__(name, id, src, unit, row)
        self.main_tab = 10     # Unique tab name
        self.sub_tab = 10     # Unique tab name
        self.parent = ''
        self.parent_id = ''


    def id(self):
        return self.id


    def set_parent(self, parent):
        self.parent = parent


    def set_parent_id(self, parent_id):
        self.parent_id = parent_id


    def get_components(self):
        return(self.name, self.id, self.src, self.unit, self.row)


    def row(self):
        '''
        Returns the row (starting from 0) of the selected item.
        '''
        return self.row

    def setSelectable(self):
        pass

    def select(self):
        self.setSelected(True)

    def deselect(self):
        self.setSelected(False)


    def disable(self):
        self.setFlags(Qt.NoItemFlags)


    def enable(self):
        self.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)


        
class ParentListItem(ListItem):
    def __init__(self, name, id, src, unit, row):
        super(ParentListItem, self).__init__(name, id, src, unit, row)
        self.setFlags(Qt.NoItemFlags)

    def setSelectable(self):
        self.setFlags(Qt.NoItemFlags)
        pass

    def select(self):
        self.setSelected(False)

    def deselect(self):
        self.setSelected(False)
    

    def enable(self):
        pass


class MasterList(List):
    def __init__(self, name, parent=None):
        super().__init__(name, parent)
        self.setDefaultDropAction(QtCore.Qt.CopyAction)
        self.data_items = []

    def dragLeaveEvent(self, event):
        #print("MasterList dragLeaveEvent")
        pass

    def dragEnterEvent(self, event):
        #print(f"MasterList dragEnterEvent ({event.mimeData()})")
        md = event.mimeData()
        #print(f"md = {md}, text = '{md.text()}'")
        event.mimeData().setText(self.objectName())
        bytearray = event.mimeData().data('application/x-qabstractitemmodeldatalist')
        self.data_items = get_rows(bytearray)
        #print(f"data_items = {self.data_items}")
        #for i in self.data_items:
            #self.item(i).setFlags(Qt.NoItemFlags)
        #super().dragEnterEvent(event)
        #event.accept() # Enables the dragLeaveEvent method for some stupid reason

    def dropEvent(self, event):
        #print("MasterList dropEvent")
        super().dropEvent(event)

